#include "stdafx.h"
#include <stdio.h>
#include <stdlib.h>
#include "MyWindow.h"
#include "MyApplication.h"


//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyWindow::CMyWindow(LPCWSTR pwzClassTemplate) : CMyWindowObject()
{
	Init(pwzClassTemplate, 0, (WNDPROC)WindowProc);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyWindow::CMyWindow(UINT nClassTemplate) : CMyWindowObject()
{
	Init(NULL, nClassTemplate, (WNDPROC)WindowProc);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyWindow::CMyWindow(LPCWSTR pwzClassTemplate, WNDPROC lpWindowProc) : CMyWindowObject()
{
	Init(pwzClassTemplate, 0, lpWindowProc);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyWindow::CMyWindow(UINT nClassTemplate, WNDPROC lpWindowProc) : CMyWindowObject()
{
	Init(NULL, nClassTemplate, lpWindowProc);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyWindow::CMyWindow(UINT nClassTemplate, WNDPROC lpWindowProc, HWND hParentWnd) : CMyWindowObject()
{
	Init(NULL, nClassTemplate, lpWindowProc, hParentWnd);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyWindow::~CMyWindow()
{
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CMyWindow::Init(LPCWSTR pwzClassTemplate, UINT nClassTemplate, WNDPROC lpWindowProc, HWND hParentWnd)
{
	m_nMenu							= 0;
	m_nSmallIcon					= 0;
	m_nBigIcon						= 0;
	m_pParentWindow				= NULL;
	m_nClassTemplate				= nClassTemplate;
	m_bCreate						= FALSE;
	m_lpWindowProc					= lpWindowProc;
	m_lpOriginalWindowProc		= NULL;
	m_hWnd							= NULL;
	m_hParentWnd					= hParentWnd;
	
	if (pwzClassTemplate != NULL) wcscpy_s(m_wzClassTemplate, _countof(m_wzClassTemplate), pwzClassTemplate);   
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
LRESULT CALLBACK CMyWindow::WindowProc(HWND hWnd, UINT nMessage, WPARAM wParam, LPARAM lParam)
{
	CMyWindow*	pThis = NULL;	

	if ((nMessage == WM_CREATE) || (nMessage == WM_NCCREATE)) {
		LPCREATESTRUCT	psCreate = (LPCREATESTRUCT)lParam;
		if (psCreate != NULL) pThis = (CMyWindow*)psCreate->lpCreateParams;
	}
	else pThis = (CMyWindow*)::GetWindowLong(hWnd, GWL_USERDATA);

	if (pThis == NULL) return(DefWindowProc(hWnd, nMessage, wParam, lParam));	

	switch(nMessage) {
		case(WM_COMMAND):
			return(pThis->OnCommand(wParam, lParam));
		case(WM_CREATE):
			::SetWindowLong(hWnd, GWL_USERDATA, (LPARAM)pThis);
			pThis->SaveHandle(hWnd);
			return(pThis->OnCreate((LPCREATESTRUCT)lParam));
		case(WM_DESTROY): {
			BOOL	bRet = pThis->OnDestroy();
			// deinicializace atributu
			pThis->Destroy();

			return(bRet);
		}
		case(WM_NCCREATE):
			::SetWindowLong(hWnd, GWL_USERDATA, (LPARAM)pThis);
			pThis->SaveHandle(hWnd);
			return(pThis->OnNCCreate((LPCREATESTRUCT)lParam));
	};	

	return(pThis->OnOtherMessage(nMessage, wParam, lParam));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::OnCommand(WPARAM wParam, LPARAM lParam)
{
	if (m_lpOriginalWindowProc != NULL) return(CallWindowProc(m_lpOriginalWindowProc, m_hWnd, WM_COMMAND, wParam, lParam));
	else return(DefWindowProc(m_hWnd, WM_COMMAND, wParam, lParam));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::OnCreate(LPCREATESTRUCT psCreateStruct)
{
	if (m_lpOriginalWindowProc != NULL) return(CallWindowProc(m_lpOriginalWindowProc, m_hWnd, WM_CREATE, (WPARAM)NULL, (LPARAM)psCreateStruct));
	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::OnDestroy()
{
	if (m_lpOriginalWindowProc != NULL) return(CallWindowProc(m_lpOriginalWindowProc, m_hWnd, WM_DESTROY, (WPARAM)0, (LPARAM)0));
	else return(DefWindowProc(m_hWnd, WM_DESTROY, (WPARAM)0, (LPARAM)0));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::OnNCCreate(LPCREATESTRUCT psCreateStruct)
{
	if (m_lpOriginalWindowProc != NULL) return(CallWindowProc(m_lpOriginalWindowProc, m_hWnd, WM_NCCREATE, (WPARAM)NULL, (LPARAM)psCreateStruct));
	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
LRESULT CMyWindow::OnOtherMessage(UINT nMessage, WPARAM wParam, LPARAM lParam)
{
	// ostatni nezpracovane zpravy se musi preposlat originalni 
	if (m_lpOriginalWindowProc != NULL) return(CallWindowProc(m_lpOriginalWindowProc, m_hWnd, nMessage, wParam, lParam));
	else return(DefWindowProc(m_hWnd, nMessage, wParam, lParam));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::Create(LPCWSTR pwzTitle, DWORD dwStyle, HWND hParentWnd, UINT nMenu,
							  UINT nBigIcon, UINT nSmallIcon, RECT* pPosition)
{
	HINSTANCE	hInstance;
	WNDCLASSEX	sClassInfoEx;

	if (m_bCreate) {
		OutputDebugString(L"Okno je jiz vytvoreno.\n");
		return(TRUE);
	}

	if ((wcslen(m_wzClassTemplate) < 1) && (m_nClassTemplate == 0)) {
		OutputDebugString(L"ClassTemplate je NULL (0)!\n");
		assert(0);
		return(FALSE);
	}

	if (m_pApplication == NULL) {
		OutputDebugString(L"Aplikace je NULL.\n");
		assert(0);
		return(FALSE);
	}

	hInstance		= m_pApplication->GetSaveInstance();
	m_hParentWnd	= hParentWnd;
	m_nMenu			= nMenu;
	m_nBigIcon		= nBigIcon;
	m_nSmallIcon	= nSmallIcon;

	if (!GetClassInfoEx(hInstance, (wcslen(m_wzClassTemplate) < 1) ? MAKEINTRESOURCE(m_nClassTemplate) : m_wzClassTemplate, &sClassInfoEx)) {
		if (!RegisterMyWindow(hInstance)) {
			OutputDebugString(L"Registrace okna se nezdarila!.\n");
			assert(0);
			return(FALSE);
		}
	}

	if (!CreateMyWindow(hInstance, pwzTitle, dwStyle, pPosition, 0))
		return(FALSE);

	m_bCreate = TRUE;

	if (GetWindowLong(GWL_USERDATA) != (LONG)this)
		SetWindowLong(GWL_USERDATA, (LONG)this);

	if (GetWindowLong(GWL_WNDPROC) != (LONG)WindowProc) {
		if (m_lpOriginalWindowProc != NULL) assert(0);
		m_lpOriginalWindowProc = (WNDPROC)SetWindowLong(GWL_WNDPROC, (LONG)WindowProc);
	}

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::CreateEx(LPCWSTR pwzTitle, DWORD dwStyle, DWORD dwStyleEx, HWND hParentWnd, UINT nMenu,
													UINT nBigIcon, UINT nSmallIcon, RECT* pPosition)
{
	HINSTANCE 	hInstance;
	WNDCLASSEX	sClassInfoEx;

	if (m_bCreate) {
		OutputDebugString(L"Okno je jiz vytvoreno.\n");
		return(TRUE);
	}

	if ((wcslen(m_wzClassTemplate) < 1) && (m_nClassTemplate == 0)) {
		OutputDebugString(L"ClassTemplate je NULL (0)!\n");
		assert(0);
		return(FALSE);
	}

	if (m_pApplication == NULL) {
		OutputDebugString(L"Aplikace je NULL.\n");
		assert(0);
		return(FALSE);
	}
	hInstance		= m_pApplication->GetSaveInstance();
	m_hParentWnd	= hParentWnd;
	m_nMenu			= nMenu;
	m_nBigIcon		= nBigIcon;
	m_nSmallIcon	= nSmallIcon;
	
	if (!GetClassInfoEx(hInstance, (wcslen(m_wzClassTemplate) < 1) ? MAKEINTRESOURCE(m_nClassTemplate) : m_wzClassTemplate, &sClassInfoEx)) {
		if (!RegisterMyWindow(hInstance)) {
			OutputDebugString(L"Registrace okna se nezdarila!.\n");
			assert(0);
			return(FALSE);
		}
	}

	if (!CreateMyWindowEx(hInstance, pwzTitle, dwStyle, dwStyleEx, pPosition, (nMenu == 0) ? NULL : (HMENU)MAKEINTRESOURCE(nMenu)))
		return(FALSE);

	m_bCreate = TRUE;

	if (GetWindowLong(GWL_USERDATA) != (LONG)this)
		SetWindowLong(GWL_USERDATA, (LONG)this);

	if (GetWindowLong(GWL_WNDPROC) != (LONG)WindowProc) {
		if (m_lpOriginalWindowProc != NULL) assert(0);
		m_lpOriginalWindowProc = (WNDPROC)SetWindowLong(GWL_WNDPROC, (LONG)WindowProc);
	}

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::CreateMy(LPCWSTR pwzTitle, DWORD dwStyle, CMyWindow* pParentWindow, UINT nMenu,
							  UINT nBigIcon, UINT nSmallIcon, RECT* pPosition)
{
	HINSTANCE	hInstance;
	WNDCLASSEX	sClassInfoEx;
	HWND			hParentWnd;

	if (m_bCreate) {
		OutputDebugString(L"Okno je jiz vytvoreno.\n");
		return(TRUE);
	}

	if ((pParentWindow == NULL) || ((hParentWnd = pParentWindow->GetSafeHandle()) == NULL)) {
		OutputDebugString(L"pParentWindow je NULL!\n");
		assert(0);
		return(FALSE);
	}

	if ((wcslen(m_wzClassTemplate) < 1) && (m_nClassTemplate == 0)) {
		OutputDebugString(L"ClassTemplate je NULL (0)!\n");
		assert(0);
		return(FALSE);
	}

	if (m_pApplication == NULL) {
		OutputDebugString(L"Aplikace je NULL.\n");
		assert(0);
		return(FALSE);
	}

	hInstance		= m_pApplication->GetSaveInstance();
	m_hParentWnd	= hParentWnd;
	m_nMenu			= nMenu;
	m_nBigIcon		= nBigIcon;
	m_nSmallIcon	= nSmallIcon;

	if (!GetClassInfoEx(hInstance, (wcslen(m_wzClassTemplate) < 1) ? MAKEINTRESOURCE(m_nClassTemplate) : m_wzClassTemplate, &sClassInfoEx)) {
		if (!RegisterMyWindow(hInstance)) {
			OutputDebugString(L"Registrace okna se nezdarila!.\n");
			assert(0);
			return(FALSE);
		}
	}

	if (!CreateMyWindow(hInstance, pwzTitle, dwStyle, pPosition, (HMENU)nMenu))
		return(FALSE);

	m_bCreate			= TRUE;
	m_pParentWindow	= pParentWindow;

	if (GetWindowLong(GWL_USERDATA) != (LONG)this)
		SetWindowLong(GWL_USERDATA, (LONG)this);

	if (GetWindowLong(GWL_WNDPROC) != (LONG)WindowProc) {
		if (m_lpOriginalWindowProc != NULL) assert(0);
		m_lpOriginalWindowProc = (WNDPROC)SetWindowLong(GWL_WNDPROC, (LONG)WindowProc);
	}

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::CreateMyEx(LPCWSTR pwzTitle, DWORD dwStyle, DWORD dwStyleEx, CMyWindow* pParentWindow, UINT nMenu,
													UINT nBigIcon, UINT nSmallIcon, RECT* pPosition)
{
	HINSTANCE 	hInstance;
	WNDCLASSEX	sClassInfoEx;
	HWND			hParentWnd;

	if (m_bCreate) {
		OutputDebugString(L"Okno je jiz vytvoreno.\n");
		return(TRUE);
	}

	if ((wcslen(m_wzClassTemplate) < 1) && (m_nClassTemplate == 0)) {
		OutputDebugString(L"ClassTemplate je NULL (0)!\n");
		assert(0);
		return(FALSE);
	}

	if ((pParentWindow == NULL) || ((hParentWnd = pParentWindow->GetSafeHandle()) == NULL)) {
		OutputDebugString(L"pParentWindow je NULL!\n");
		assert(0);
		return(FALSE);
	}

	if (m_pApplication == NULL) {
		OutputDebugString(L"Aplikace je NULL.\n");
		assert(0);
		return(FALSE);
	}
	hInstance		= m_pApplication->GetSaveInstance();
	m_hParentWnd	= hParentWnd;
	m_nMenu			= nMenu;
	m_nBigIcon		= nBigIcon;
	m_nSmallIcon	= nSmallIcon;
	
	if (!GetClassInfoEx(hInstance, (wcslen(m_wzClassTemplate) < 1) ? MAKEINTRESOURCE(m_nClassTemplate) : m_wzClassTemplate, &sClassInfoEx)) {
		if (!RegisterMyWindow(hInstance)) {
			OutputDebugString(L"Registrace okna se nezdarila!.\n");
			assert(0);
			return(FALSE);
		}
	}

	if (!CreateMyWindowEx(hInstance, pwzTitle, dwStyle, dwStyleEx, pPosition, (nMenu == 0) ? NULL : (HMENU)MAKEINTRESOURCE(nMenu)))
		return(FALSE);

	m_bCreate			= TRUE;
	m_pParentWindow	= pParentWindow;

	if (GetWindowLong(GWL_USERDATA) != (LONG)this)
		SetWindowLong(GWL_USERDATA, (LONG)this);

	if (GetWindowLong(GWL_WNDPROC) != (LONG)WindowProc) {
		if (m_lpOriginalWindowProc != NULL) assert(0);
		m_lpOriginalWindowProc = (WNDPROC)SetWindowLong(GWL_WNDPROC, (LONG)WindowProc);
	}

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::RegisterMyWindow(HINSTANCE hInstance)
{
	WNDCLASSEX  sWndClassEx;

	if (hInstance == NULL) return(FALSE);

	sWndClassEx.cbSize         = sizeof(WNDCLASSEX);
	sWndClassEx.style			   = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	sWndClassEx.lpfnWndProc	   = (m_lpWindowProc == NULL) ? (WNDPROC)WindowProc : (WNDPROC)m_lpWindowProc;
	sWndClassEx.cbClsExtra		= 0;
	sWndClassEx.cbWndExtra		= 0;
	sWndClassEx.hInstance		= hInstance;	
	sWndClassEx.hIcon			   = (m_nBigIcon == 0) ? NULL : LoadIcon(hInstance, MAKEINTRESOURCE(m_nBigIcon));
	sWndClassEx.hCursor		   = LoadCursor(NULL, IDC_ARROW);
	sWndClassEx.hbrBackground	= (HBRUSH)(COLOR_WINDOW + 1);
	sWndClassEx.lpszMenuName	= (m_nMenu == 0) ? NULL : MAKEINTRESOURCE(m_nMenu);
	sWndClassEx.lpszClassName	= (wcslen(m_wzClassTemplate) < 1) ? MAKEINTRESOURCE(m_nClassTemplate) : m_wzClassTemplate;
	sWndClassEx.hIconSm		   = (m_nSmallIcon == 0) ? NULL : LoadIcon(hInstance, MAKEINTRESOURCE(m_nSmallIcon));

	return(RegisterClassEx(&sWndClassEx));
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::CreateMyWindowEx(HINSTANCE hInstance, LPCWSTR pwzTitle, DWORD dwStyle, DWORD dwStyleEx, RECT* pPosition, HMENU hMenu)
{
	RECT rtRect = {CW_USEDEFAULT, 0, CW_USEDEFAULT, 0};

	if ((hInstance == NULL) || (pwzTitle == NULL)) {
		OutputDebugString(L"HINSTANCE a/nebo titulek je NULL!\n");
		assert(0);
		return(FALSE);
	}

	if (pPosition != NULL) memcpy(&rtRect, pPosition, sizeof(RECT));

	m_hWnd = CreateWindowEx(dwStyleEx, (wcslen(m_wzClassTemplate) < 1) ? MAKEINTRESOURCE(m_nClassTemplate) : m_wzClassTemplate, pwzTitle,
					dwStyle, rtRect.left, rtRect.top, rtRect.right - rtRect.left, rtRect.bottom - rtRect.top, m_hParentWnd, hMenu,
					hInstance, this);

	if (m_hWnd == NULL) {
		DWORD		dwError = GetLastError();
		wchar_t	wzTemp[32];
		OutputDebugString(L"CreateWindow vratil NULL s chybou ");
		swprintf_s(wzTemp, 32, L"%d\n", dwError);
		OutputDebugString(wzTemp);		
		return(FALSE);
	}		

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
BOOL CMyWindow::CreateMyWindow(HINSTANCE hInstance, LPCWSTR pwzTitle, DWORD dwStyle, RECT* pPosition, HMENU hMenu)
{
	RECT rtRect = {CW_USEDEFAULT, 0, CW_USEDEFAULT, 0};

	if (hInstance == NULL) hInstance = (HINSTANCE)::GetWindowLong(m_hParentWnd, GWL_HINSTANCE);

	if ((hInstance == NULL) || (pwzTitle == NULL)) {
		OutputDebugString(L"HINSTANCE a/nebo titulek je NULL!\n");
		assert(0);
		return(FALSE);
	}

	if (pPosition != NULL) memcpy(&rtRect, pPosition, sizeof(RECT));

	m_hWnd = CreateWindow((wcslen(m_wzClassTemplate) < 1) ? MAKEINTRESOURCE(m_nClassTemplate) : m_wzClassTemplate, pwzTitle,
					dwStyle, rtRect.left, rtRect.top, rtRect.right - rtRect.left, rtRect.bottom - rtRect.top, m_hParentWnd, hMenu,
					hInstance, this);

	if (m_hWnd == NULL) {
		DWORD		dwError = GetLastError();
		wchar_t	wzTemp[32];
		OutputDebugString(L"CreateWindow vratil NULL s chybou ");
		swprintf_s(wzTemp, 32, L"%d\n", dwError);
		OutputDebugString(wzTemp);	
		return(FALSE);
	}	

	return(TRUE);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
HINSTANCE CMyWindow::GetSafeInstance()
{
	if (this == NULL) return(NULL);
	if (m_pApplication == NULL) return(NULL);
	return(m_pApplication->GetSaveInstance());
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyData* CMyWindow::GetSaveData()
{
	if (this == NULL) return(NULL);
	if (m_pApplication == NULL) return(NULL);	
	return(m_pApplication->GetSaveData());
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
CMyWindow* CMyWindow::GetParentWindow()
{
	return(m_pParentWindow);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
LPVOID CMyWindow::GetWndProc()
{
	return(m_lpWindowProc);
}

//- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
void CMyWindow::Destroy()
{
	m_bCreate						= FALSE;	
	m_pParentWindow				= NULL;	
	m_nMenu							= 0;
	m_nSmallIcon					= 0;
	m_nBigIcon						= 0;
	m_lpOriginalWindowProc		= NULL;
	m_hWnd							= NULL;
	m_hParentWnd					= NULL;
}
