#ifndef MyWindowH
#define MyWindowH

#include <windows.h>
#include "MyWindowObject.h"
#include <ShellApi.h>


class CMyData;


/** Tda CMyWindow ja zkladni tdou vech okennch a dialogovch td
	(pes CMyDialog) a td ovldacch prvk. Obsahuje 'errn' zprvovou
	funkci,kterou lze jednodue (v konstruktoru dcein tdy) nahradit za
	vlastn.	Jinak je pouita errn, kter vol virtuln funkce odpovdajc 
	systmovm zprvm Window tzn., e nap. pi WM_CLOSE je volna funkce
	OnClose atd. Dle tdy obsahuje vechny spolen prvky pro okna,
	dialogy a ovldac prvky (handle okna, handle rodiovskho okna,
	adresu zpvov funkce, identifiktory ikon a menu atd.). Tak je
	k dispozici statick atribut, kterm je pointer na aplikaci tzn., e
	vechny instance dceinch td budou vdt v jak aplikaci jsou
	sputny. To umouje napklad manipulaci s daty aplikace apod.
	
	Tda dle e funknost zkladnch operac jako je zavrn okna,
	nastaven textu, icony a menu. Obsahuje tak vtinu (asem vechny)
	funkce uren pro manipulaci s okny. 

	Typick pklad vytvoen nedialogovho okna ve funkci ShowApplicationWindow

	m_pWindow->Create("Ahoj", WS_OVERLAPPEDWINDOW | WS_VISIBLE, NULL);	
	return(m_pWindow->Show());

*/
class __declspec(dllexport) CMyWindow : public CMyWindowObject {
//Konstruktory a destruktor
public:	
	/** Destruktor*/
	virtual ~CMyWindow();

protected:
	/** Konstruktor.<BR>
		Konstruktor tdy CMyWindow.<BR>
		@param pwzClassTemplate ... etzec ukonen nulovm znakem, kterm
			obsahuje uniktn jmno tdy okna (pro registraci), nesm bt NULL*/
	CMyWindow(LPCWSTR pwzClassTemplate);
	/** Konstruktor.<BR>
		Konstruktor tdy CMyWindow.<BR>
		@param nClassTemplate ... identifiktor uniktnho jmna tdy okna
			(nebo dialogovho okna)*/
	CMyWindow(UINT nClassTemplate);
	/** Konstruktor.<BR>
		Konstruktor tdy CMyWindow.<BR>
		@param pwzClassTemplate ... etzec ukonen nulovm znakem, kterm
			obsahuje uniktn jmno tdy (pro registraci), nesm bt NULL
		@param lpWindowProc ... adresa hlavn okenn funkce pro pjem
			zprv, pokud je NULL pouije se implicitn funkce */
	CMyWindow(LPCWSTR pwzClassTemplate, WNDPROC lpWindowProc);
	/** Konstruktor.<BR>
		Konstruktor tdy CMyWindow.<BR>
		@param nClassTemplate ... identifiktor uniktnho jmna tdy okna 
			(nebo dialogovho okna)
		@param lpWindowProc ... adresa hlavn okenn funkce pro pjem
			zprv, pokud je NULL pouije se implicitn funkce */
	CMyWindow(UINT nClassTemplate, WNDPROC lpWindowProc);
	/** Konstruktor.<BR>
		Konstruktor tdy CMyWindow.<BR>
		@param nClassTemplate ... identifiktor uniktnho jmna tdy okna 
			(nebo dialogovho okna)
		@param lpWindowProc ... adresa hlavn okenn funkce pro pjem
			zprv, pokud je NULL pouije se implicitn funkce
		@param hParentWnd ... handle rodiovskho okna*/
	CMyWindow(UINT nClassTemplate, WNDPROC lpWindowProc, HWND hParentWnd);

private:



//Funkce a metody
public:	
	/** Vytvoen okna */
	virtual BOOL					Create(LPCWSTR pwzTitle, DWORD dwStyle, HWND hParentWnd, UINT nMenu = 0,
													UINT nBigIcon = 0, UINT nSmallIcon = 0, RECT* pPosition = NULL);
	/** Vytvoen okna */
	virtual BOOL					CreateEx(LPCWSTR pwzTitle, DWORD dwStyle, DWORD dwStyleEx, HWND hParentWnd, UINT nMenu = 0,
													UINT nBigIcon = 0, UINT nSmallIcon = 0, RECT* pPosition = NULL);
	/** Vytvoen okna */
	virtual BOOL					CreateMy(LPCWSTR pwzTitle, DWORD dwStyle, CMyWindow* pParentWindow, UINT nMenu = 0,
													UINT nBigIcon = 0, UINT nSmallIcon = 0, RECT* pPosition = NULL);
	/** Vytvoen okna */
	virtual BOOL					CreateMyEx(LPCWSTR pwzTitle, DWORD dwStyle, DWORD dwStyleEx, CMyWindow* pParentWindow, UINT nMenu = 0,
													UINT nBigIcon = 0, UINT nSmallIcon = 0, RECT* pPosition = NULL);	


protected:
	/** Zskn pointeru na data */
	CMyData*							GetSaveData();
	/** Zskn HINSTANCE */
	HINSTANCE						GetSafeInstance();
	/** Zskn adresy WindowProc */
	LPVOID							GetWndProc();	
	/** Zskn pointeru na rodiovsk okno */
	CMyWindow*						GetParentWindow();
	/** Prkaz od komponenty.<BR>
		Reakce na WM_COMMAND, prkaz od komponenty.<BR>
		@param wParam ... WPARAM prkazu (typicky LOWORD je ID komponenty a HIWORD
			je typ provedenho prkazu)
		@param lParam ... LPARAM prkazu (typicky HWND komponenty) */
	virtual BOOL					OnCommand(WPARAM wParam, LPARAM lParam);	
	/** Vytvoen okna.<BR>
		Reakce na WM_CREATE, vytvoen okna.<BR>
		@param psCreateStruct ... struktura obsahujc informace o vytvoen okna */
	virtual BOOL					OnCreate(LPCREATESTRUCT psCreateStruct);
	/** Zruen okna.<BR>
		Reakce na WM_DESTROY, zruen okna.<BR>*/
	virtual BOOL					OnDestroy();	
	/** Vytvoen neklienstk sti okna.<BR>
		Reakce na WM_NCCREATE, vytvoen neklienstk sti okna.<BR>
		@param psCreateStruct ... struktura obsahujc informace o vytvoen okna*/
	virtual BOOL					OnNCCreate(LPCREATESTRUCT psCreateStruct);
	/** Ostatn zprvy.<BR>
		Reakce na ostatn zprvy nejastji WM_USER apod.<BR>
		@param nMessage ... slo zprvy
		@param wParam ... WPARAM zprvy
		@param lParam ... LPARAM zprvy */
	virtual LRESULT				OnOtherMessage(UINT nMessage, WPARAM wParam, LPARAM lParam);	
	
private:
	/** Inicializace atribut. */
	void								Init(LPCWSTR pwzClassTemplate, UINT nClassTemplate, WNDPROC lpWindowProc, HWND hParentWnd = NULL);
	/** Registrace tdy okna. */
	virtual BOOL					RegisterMyWindow(HINSTANCE hInstance);
	/** Vytvoen okna. */
	BOOL								CreateMyWindow(HINSTANCE hInstance, LPCWSTR pwzTitle, DWORD dwStyle, RECT* pPosition, HMENU hMenu = NULL);
	/** Vytvoen okna. */
	BOOL								CreateMyWindowEx(HINSTANCE hInstance, LPCWSTR pwzTitle, DWORD dwStyle, DWORD dwStyleEx, RECT* pPosition, HMENU hMenu = NULL);
	/** Deinicializace atribut. */	
	void								Destroy();
	/** WindowProc */
	static LRESULT CALLBACK		WindowProc(HWND hWnd, UINT nMessage, WPARAM wParam, LPARAM lParam);

	

//Atributy a datove cleny
public:

protected:
	/** HWND rodiovskho okna. */
	HWND								m_hParentWnd;

private:
	/** Nae WindowProc */
	WNDPROC							m_lpWindowProc;
	/** Pvodn WindowProc */
	WNDPROC							m_lpOriginalWindowProc;
	/** ID menu */
	UINT								m_nMenu;
	/** ID mal ikony */
	UINT								m_nSmallIcon;
	/** ID velk ikony */
	UINT								m_nBigIcon;
	/** ID tdy okna */
	UINT								m_nClassTemplate;
	/** Nzev tdy okna */
	wchar_t							m_wzClassTemplate[128];	
	/** Flag pro oznaen vytvoenho okna */
	BOOL								m_bCreate;
	/** Rodiovsk okna (CMyWindow) */
	CMyWindow*						m_pParentWindow;		


friend class CMyApplication;

}; 

#endif


//Vygenerovno prostrednictvm ClassGen 1.00.002